@@ -3,7 +3,7 @@
 """
 pywurfl Browser Example
 
-Copyright 2004-2006, Armand Lynch <lyncha@users.sourceforge.net>
+Copyright 2004-2010, Armand Lynch <lyncha@users.sourceforge.net>
 
 This code is free software; you can redistribute it and/or modify it under
 the terms of the LGPL License.
@@ -36,7 +36,7 @@ def get_info():
     if type_var.get() == "User Agent":
         try:
             device = devices.select_ua(ua, actual_device_root=adr_var.get(),
-                                       filter_noise=filter_var.get(),
+                                       normalize=normalize_var.get(),
                                        search=get_algo())
         except DeviceNotFound:
             showerror('Error', "Device Not Found")
@@ -191,7 +191,7 @@ mfile.add_command(label="Exit", command=root.quit)
 
 type_var = StringVar()
 stype_var = StringVar()
-filter_var = BooleanVar()
+normalize_var = BooleanVar()
 adr_var = BooleanVar()
 
 type_var.set("User Agent")
@@ -199,8 +199,8 @@ if JAROWINKLER:
     stype_var.set("Jaro-Winkler")
 else:
     stype_var.set("Tokenizer")
-# filter_noise
-filter_var.set(True)
+# normalize_noise
+normalize_var.set(True)
 # actual_device_root
 adr_var.set(False)
 
@@ -210,7 +210,7 @@ soptions.add_radiobutton(label="User Agent", variable=type_var,
 soptions.add_radiobutton(label="WURFL ID", variable=type_var,
                          command=change_type)
 soptions.add_separator()
-soptions.add_checkbutton(label="Filter Noise", variable=filter_var)
+soptions.add_checkbutton(label="Normalize User Agent", variable=normalize_var)
 soptions.add_checkbutton(label="Actual Device", variable=adr_var)
 if JAROWINKLER:
     soptions.add_separator()
