@@ -28,6 +28,7 @@ from copy import copy
 
 from pywurfl.exceptions import (WURFLException, ActualDeviceRootNotFound,
                                 DeviceNotFound, ExistsException)
+from pywurfl.algorithms.wurfl.normalizers import default_normalizer
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
@@ -35,7 +36,7 @@ __contributors__ = "Pau Aliagas <pau@newtral.org>"
 __copyright__ = "Copyright 2006-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "7.0.0"
+__version__ = "7.1.0"
 __all__ = ['devclass', 'Devices']
 
 
@@ -124,13 +125,6 @@ class Devices(object):
     def __init__(self):
         self.devids = {}
         self.devuas = {}
-
-        # Regexes to remove noise from user agents
-        self._noise = ((u"/SN\d{15}", u"/SNXXXXXXXXXXXXXXX"),
-                       (u"UP.Link\/.*?(\s|$)", u""),
-                       (u"\(via IBM WBI \d+\.\d+\)", u""),
-                       (u"GMCC\/\d\.\d", u""))
-        self._noise_re = [(re.compile(exp), repl) for exp, repl in self._noise]
         self._name_test_re = re.compile(ur'^(_|[a-z])(_|[a-z]|[0-9])+$')
 
     def find_actual_root(self, device=RootDevice):
@@ -147,7 +141,7 @@ class Devices(object):
             device = device.__bases__[0]
         raise ActualDeviceRootNotFound
 
-    def select_ua(self, devua, actual_device_root=False, filter_noise=True,
+    def select_ua(self, devua, actual_device_root=False, normalize=True,
                   search=None, instance=True):
         """
         Return a Device object based on the user agent.
@@ -157,8 +151,8 @@ class Devices(object):
         @param actual_device_root: Return a device that is an actual device
                                    root
         @type actual_device_root: boolean
-        @param filter_noise: Remove noise words from devua.
-        @type filter_noise: boolean
+        @param normalize: Normalize the user agent by removing extraneous text.
+        @type normalize: boolean
         @param search: The algorithm to use for searching. If 'search' is None,
                        a search will not be performed.
         @type search: pywurfl.Algorithm
@@ -176,10 +170,8 @@ class Devices(object):
                 device = self.find_actual_root(device)
 
         if device is None:
-            if filter_noise:
-                for exp, repl in self._noise_re:
-                    devua = exp.sub(repl, devua)
-                devua = devua.strip()
+            if normalize:
+                devua = default_normalizer(devua)
 
                 if devua in self.devuas:
                     device = self.devuas.get(devua)
