@@ -1,5 +1,5 @@
 # pywurfl QL - Wireless Universal Resource File Query Language in Python
-# Copyright (C) 2006-2009 Armand Lynch
+# Copyright (C) 2006-2010 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -136,43 +136,43 @@ Example Queries
 EBNF
 ====
 
-query := select_statement where_statement
+query ::= select_statement where_statement
 
-select_statement := 'select' ('device' | 'id' | 'ua')
+select_statement ::= 'select' ('device' | 'id' | 'ua')
 
-where_statement := 'where' + where_expression
+where_statement ::= 'where' + where_expression
 
-where_expression := where_test (boolop where_test)*
+where_expression ::= where_test (boolop where_test)*
 
-where_test := (any_statement | all_statement | expr_test)
+where_test ::= (any_statement | all_statement | expr_test)
 
-any_statement := 'any' '(' expr_list ')' operator expr
+any_statement ::= 'any' '(' expr_list ')' operator expr
 
-all_statement := 'all' '(' expr_list ')' operator expr
+all_statement ::= 'all' '(' expr_list ')' operator expr
 
-capability := alphanums ('_' alphanums)*
+capability ::= alphanums ('_' alphanums)*
 
-expr_test := expr operator expr
+expr_test ::= expr operator expr
 
-expr_list := expr (',' expr)*
+expr_list ::= expr (',' expr)*
 
-expr := types attributes_methods_concat | capability attributes_methods_concat
+expr ::= types attributes_methods_concat | capability attributes_methods_concat
 
-attributes_methods_concat := ('.' method '(' method_args? ')')*
+attributes_methods_concat ::= ('.' method '(' method_args? ')')*
 
-method_args := (method_arg (',' method_arg)*)
+method_args ::= (method_arg (',' method_arg)*)
 
-method_arg := (types | expr)
+method_arg ::= (types | expr)
 
-method := ('_' alphanums)*
+method ::= ('_' alphanums)*
 
-operator := ('='|'!='|'<'|'>'|'>='|'<=')
+operator ::= ('='|'!='|'<'|'>'|'>='|'<=')
 
-types := (<quote> string <quote> | integer | boolean)
+types ::= (<quote> string <quote> | integer | boolean)
 
-boolean := ('true' | 'false')
+boolean ::= ('true' | 'false')
 
-boolop := ('and' | 'or')
+boolop ::= ('and' | 'or')
 """
 
 import re
@@ -187,7 +187,7 @@ from pywurfl.exceptions import WURFLException
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Gabriele Fantini <gabriele.fantini@staff.dada.net>"
-__copyright__ = "Copyright 2006-2009, Armand Lynch"
+__copyright__ = "Copyright 2006-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['QueryLanguageError', 'QL']
@@ -293,8 +293,7 @@ def define_language():
     id_token = CaselessKeyword("id")
     device_token = CaselessKeyword("device")
     select_type = (device_token | ua_token | id_token)("type")
-    select_clause = select_token + select_type
-    select_statement = Group(select_clause)("select")
+    select_statement = (select_token + select_type)("select")
 
     expr = Forward()
 
@@ -340,12 +339,6 @@ def define_language():
     where_expression = Forward()
     where_expression << Group(where_test + ZeroOrMore(boolop + where_expression))('where_expression')
 
-    #where_expression << (Group(where_test + ZeroOrMore(boolop +
-    #                                                   where_expression) +
-    #                           StringEnd())('where'))
-    #where_expression = (Group(where_test + ZeroOrMore(boolop + where_test) +
-    #                    StringEnd())('where'))
-
     where_statement = where_token + where_expression
 
     # Mon Jan  1 12:35:56 EST 2007
@@ -402,90 +395,93 @@ def expr_test(lexpr, op, rexpr):
     """
 
     def expr_tester(devobj):
-
-        def evaluate(expression):
-            value = None
-            if expression.keys() == ['expr']:
-                expression = expression.expr
-            # check wheather the expression is a capability or not
-            if 'capability' in expression.keys():
-                capability = expression.capability
-                try:
-                    py_value = getattr(devobj, capability)
-                except AttributeError, e:
-                    raise QueryLanguageError("Invalid capability '%s'" %
-                                             capability)
-
-                if isinstance(py_value, bool):
-                    value = TypeBool(py_value)
-                elif isinstance(py_value, int):
-                    value = TypeNum(py_value)
-                elif isinstance(py_value, str):
-                    value = TypeStr(py_value)
-                else:
-                    raise QueryLanguageError("Unknown type '%s'" %
-                                             py_value.__class__)
-            else:
-                value = expression.value
-
-            for attribute in expression.attribute_concat:
-                py_value = None
-                if 'attribute_call' in attribute.keys():
-                    method_name = attribute.attribute_call.attribute
-                    method_args = []
-                    for method_arg in attribute.attribute_call.method_args:
-                        method_arg_value = None
-                        try:
-                            method_arg_value = evaluate(method_arg.expression)
-                        except AttributeError, e:
-                            method_arg_value = method_arg
-
-                        method_args.append(method_arg_value.py_value)
-
-                    try:
-                        attr = getattr(value, method_name)
-                        py_value = attr(*method_args)
-                    except (AttributeError, TypeError), e:
-                        msg = "'%s' object has no callable attribute '%s'"
-                        raise QueryLanguageError(msg %
-                                                 (type(value.py_value).__name__,
-                                                  method_name))
-                elif 'attribute' in attribute.keys():
-                    try:
-                        py_value = getattr(value, attribute.attribute)
-                    except AttributeError, e:
-                        raise QueryLanguageError(str(e))
-                    if callable(py_value):
-                        msg = "'%s' object has no attribute '%s'"
-                        raise QueryLanguageError(msg %
-                                                 (type(value.py_value).__name__,
-                                                  attribute.attribute))
-                else:
-                    raise QueryLanguageError('query syntax error')
-
-                if isinstance(py_value, bool):
-                    value = TypeBool(py_value)
-                elif py_value is None:
-                    value = TypeNone(py_value)
-                elif isinstance(py_value, int):
-                    value = TypeNum(py_value)
-                elif isinstance(py_value, str):
-                    value = TypeStr(py_value)
-                elif isinstance(py_value, (list, tuple)):
-                    value = TypeList(py_value)
-                else:
-                    raise QueryLanguageError("Unknown type '%s'" %
-                                             py_value.__class__)
-
-            return value
-
-        lvalue = evaluate(lexpr)
-        rvalue = evaluate(rexpr)
+        lvalue = _evaluate(devobj, lexpr)
+        rvalue = _evaluate(devobj, rexpr)
         return ops[op](lvalue.py_value, rvalue.py_value)
 
     return expr_tester
 
 
+def _evaluate(devobj, expression):
+    """
+    Evaluate an expression with respect to a device object
+    """
+    value = None
+    if expression.keys() == ['expr']:
+        expression = expression.expr
+    # check wheather the expression is a capability or not
+    if 'capability' in expression.keys():
+        capability = expression.capability
+        try:
+            py_value = getattr(devobj, capability)
+        except AttributeError, e:
+            raise QueryLanguageError("Invalid capability '%s'" %
+                                     capability)
+
+        if isinstance(py_value, bool):
+            value = TypeBool(py_value)
+        elif isinstance(py_value, int):
+            value = TypeNum(py_value)
+        elif isinstance(py_value, basestring):
+            value = TypeStr(py_value)
+        else:
+            raise QueryLanguageError("Unknown type '%s'" %
+                                     py_value.__class__)
+    else:
+        value = expression.value
+
+    for attribute in expression.attribute_concat:
+        py_value = None
+        if 'attribute_call' in attribute.keys():
+            method_name = attribute.attribute_call.attribute
+            method_args = []
+            for method_arg in attribute.attribute_call.method_args:
+                method_arg_value = None
+                try:
+                    method_arg_value = _evaluate(devobj, method_arg.expr)
+                except AttributeError, e:
+                    method_arg_value = method_arg
+
+                method_args.append(method_arg_value.py_value)
+
+            try:
+                attr = getattr(value, method_name)
+                py_value = attr(*method_args)
+            except (AttributeError, TypeError), e:
+                msg = "'%s' object has no callable attribute '%s'"
+                raise QueryLanguageError(msg %
+                                         (type(value.py_value).__name__,
+                                          method_name))
+        elif 'attribute' in attribute.keys():
+            try:
+                py_value = getattr(value, attribute.attribute)
+            except AttributeError, e:
+                raise QueryLanguageError(str(e))
+            if callable(py_value):
+                msg = "'%s' object has no attribute '%s'"
+                raise QueryLanguageError(msg %
+                                         (type(value.py_value).__name__,
+                                          attribute.attribute))
+        else:
+            raise QueryLanguageError('query syntax error')
+
+        if isinstance(py_value, bool):
+            value = TypeBool(py_value)
+        elif py_value is None:
+            value = TypeNone(py_value)
+        elif isinstance(py_value, int):
+            value = TypeNum(py_value)
+        elif isinstance(py_value, basestring):
+            value = TypeStr(py_value)
+        elif isinstance(py_value, (list, tuple)):
+            value = TypeList(py_value)
+        else:
+            raise QueryLanguageError("Unknown type '%s'" %
+                                     py_value.__class__)
+
+    return value
+
+
 def combine_funcs(funcs):
     """
     Combines a list of functions with binary operators.
@@ -556,11 +552,12 @@ def reduce_statement(exp):
     if exp.any_statement:
         for expr in exp.any_statement.any_expr_list:
             funcs.append(expr_test(expr, exp.operator, exp.rexpr))
-        return reduce_funcs(ops['or'], funcs)
+        func = ops['or']
     elif exp.all_statement:
         for expr in exp.all_statement.all_expr_list:
             funcs.append(expr_test(expr, exp.operator, exp.rexpr))
-        return reduce_funcs(ops['and'], funcs)
+        func = ops['and']
+    return reduce_funcs(func, funcs)
 
 
 def test_generator(ql_result):
@@ -627,6 +624,8 @@ def QL(devices):
         @type instance: boolean
         @rtype: generator
         """
+        if isinstance(qstr, str):
+            raise UnicodeError(u"query must be a unicode string")
         qstr = qstr.replace('\n', ' ').replace('\r', ' ') + '*'
         try:
             qres = language.parseString(qstr)
